# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os.path
from com.sun.star.sdbc import SQLException
from com.sun.star.uno import RuntimeException
from com.sun.star.lang import IllegalArgumentException

from libtooournoi import utils

class Classements(utils.Tournoi):
    def __init__(self, doc=None):
        super(Classements, self).__init__()
        if doc: self.doc = doc
        if self.doc.supportsService('com.sun.star.sheet.SpreadsheetDocument'):
            self.feuilles = self.doc.Sheets.getByName
            self.dbR = self.doc.DatabaseRanges
            self.plageRef = "_Partie" in self.doc.Title and "PARTIE" or "CUMUL"
            self.SER = (("1","S1"),("2","S2"),("3","S3"),("4","S4"),("5","S5"),("6","S6"),("7","S7"))
            self.CAT = (("V","Vermeils"),("D","Diamants"),("E","Espoirs"),("J","Juniors"),
                   ("C","Cadets"),("B","Benjamins"),("P","Poussins"),("M","Minimes"))

    def rafraichirClassements(self):
        '''Routine lancée à l'ouverture des classeurs classements
        pour mettre à jour les données'''
        classeur, feuilles = self.doc, self.feuilles
        afficheFede = classeur.NamedRanges.getByName("afficheFede").ReferredCells.String.lower() 
        for range in self.iterUno(self.dbR):
            if not range.Name in ("Topeurs","Tableur"): range.refresh()
        feuilles('Classement').getCellRangeByPosition(0,10,2,10).clearContents(31)
        feuilles('Classement_alpha').getCellRangeByPosition(0,10,2,10).clearContents(31)
        pA = classeur.Sheets.getByName("Classement").PrintAreas
        pA[0].EndColumn = 7
        if self.plageRef == "CUMUL":
            pA[0].EndColumn += self.infos['derpartie']*2
        pA[0].EndRow = self.dbR.getByName(self.plageRef).DataArea.EndRow
        feuilles('Classement').setPrintAreas( (pA[0],) )
        feuilles('Classement_alpha').setPrintAreas( (pA[0],) )
        feuilles('Classement').Columns.OptimalWidth = True
        feuilles('Classement_alpha').Columns.OptimalWidth = True
        feuilles('Classement').Columns.getByIndex(6).IsVisible = (afficheFede == "oui")
        feuilles('Classement_alpha').Columns.getByIndex(6).IsVisible = (afficheFede == "oui")
        feuilles('Infos').DrawPage.Forms.getByName("Formulaire").getByName("bout2").Label = "déprotéger les feuilles"
        feuilles('Stats').Columns.OptimalWidth = True

    def calculerCatSer(self):
        def tableautage(max, etiq, col, sc, sr):
            '''
            max = nombre de joueurs à afficher par catégorie/série
            etiq = tuple de tuples contenant les étiquettes de séries ou de catégories
            col = colonne contenant l'info dans la feuille "Classement"
            sc = première colonne (startcolumn) de la feuille "Stats" où copier les résultats
            sr = première ligne (startrow) de la feuille "Stats" où copier les résultats
            '''
            for k,v in etiq:
                stats = [dA[0][:8]]
                stats[0][0] = v
                x = 1
                for row in (x for x in dA[2:] if x[col] and x[col][0]==k):
                    if row[1] == stats[-1][1]:
                        row[0] = stats[-1][0]
                    elif x > max:
                        break
                    else:
                        row[0] = x
                    stats.append(row[:8])
                    x += 1
                plage = f.getCellRangeByPosition( sc, sr, sc+len(stats[0])-1, sr+len(stats)-1 )
                plage.setDataArray( tuple(tuple(s) for s in stats) )  # [[]] -> (()) pour renvoi vers UNO
                sr = sr+len(stats)+1
            return (sr, sc+len(stats[0]))

        maxSer = int(self.doc.NamedRanges.getByName("MaxSer").ReferredCells.Value)
        maxCat = int(self.doc.NamedRanges.getByName("MaxCat").ReferredCells.Value)
        xPrem = int(self.doc.NamedRanges.getByName("xPrem").ReferredCells.Value)
        f = self.feuilles("Stats")
        f.clearContents( 31 )
        f.merge(False)
        f.getCellRangeByName("A1").setString("CLASSEMENT PAR SÉRIES")
        f.getCellRangeByName("A1:H1").merge(True)
        f.getCellRangeByName("J1").setString("CLASSEMENT PAR CATÉGORIES")
        f.getCellRangeByName("J1:Q1").merge(True)
        f.getCellRangeByName("G3").setString("TOP :")
        f.getCellRangeByName("P3").setString("TOP :")
        f.getCellRangeByName("H3").setFormula("=TOP")
        f.getCellRangeByName("Q3").setFormula("=TOP")
        f.getCellRangeByName("I1").setString("mmm")

        dA = [list(x) for x in self.dbR.getByName(self.plageRef).ReferredCells.getDataArray()] # (()) -> [[]] pour traitement
        # séries
        coordSer = tableautage(maxSer, self.SER, 3, 0, 4)
        # catégories
        coordCat = tableautage(maxCat, self.CAT, 4, coordSer[1]+1, 4)
        f.copyRange(f.getCellByPosition( 9, coordCat[0]+1 ).CellAddress, f.getCellRangeByName("J1").RangeAddress)
        f.getCellByPosition(9, coordCat[0]+1 ).setString("LES REPÈRES")
        # repères
        l = []
        for row in dA[21::10]:
            row[0] = ''
            l.append(row[:8])
        if l:
            l = tuple(tuple(x) for x in l)
            f.getCellRangeByPosition(9, coordCat[0]+2, 9+len(l[0])-1, coordCat[0]+2+len(l)-1 ).setDataArray(l)
        f.copyRange(f.getCellByPosition(0, coordSer[0]+1).CellAddress, f.getCellRangeByName("A1").RangeAddress)
        f.getCellByPosition(0, coordSer[0]+1 ).setString("LES %d PREMIERS" % xPrem)
        # x premiers
        l = []
        for row in (x for x in dA[2:] if x[1] <= xPrem):
            row[0] = ''
            l.append(row[:8])
        if l:
            l = tuple(tuple(x) for x in l)
            f.getCellRangeByPosition(0, coordSer[0]+2, 0+len(l[0])-1, coordSer[0]+2+len(l)-1 ).setDataArray(l)
        f.Columns.OptimalWidth = True
        f.Rows.OptimalHeight = True

    def categLLN(self):
        offrow = 7   # nombre de lignes avant la première ligne joueur
        fcat = self.feuilles('CATÉGORIES')
        fcat.getCellRangeByPosition(0,offrow,1023,5000).clearContents( 31 )
        data = self.dbR.getByName(self.plageRef).ReferredCells.getDataArray()
        # copier les lignes d'en-tête
        fcat.getCellRangeByPosition(1,offrow-2,len(data[0])-1,offrow-2).setDataArray((data[0][1:],))
        fcat.getCellRangeByPosition(1,offrow-1,len(data[0])-1,offrow-1).setDataArray((data[1][1:],))
        # self.callXray(repr((data[0][1:],)))
        # déterminer le classement avec le rang
        data = [list(x) for x in data] # (()) -> [[]] pour traitement
        categ = self.doc.NamedRanges.categorie.ReferredCells.String
        newdata = []
        lasttop = 0
        for row in data[2:]:
            if row[4] == categ[0].upper():
                if lasttop == 0:
                    row[0] = 1
                elif row[7] == lasttop:
                    row[0] = '-'
                else:
                    row[0] = len(newdata)+1
                newdata.append(row)
                lasttop = row[7]
        if newdata:
            range = fcat.getCellRangeByPosition(0,0+offrow,len(row)-1,len(newdata)-1+offrow)
            range.setDataArray(tuple(tuple(s) for s in newdata) )  # [[]] -> (()) pour renvoi vers UNO
            range.Columns.OptimalWidth = True
            # hauteur
            range.Rows.Height = 453
            # largeur minimale
            if fcat.Columns.getByIndex(2).Width < 5500:
                fcat.Columns.getByIndex(2).Width = 5500
            pA = fcat.PrintAreas
            pA[0].EndColumn = 7
            if self.plageRef == "CUMUL":
                pA[0].EndColumn += self.infos['derpartie']*2
            pA[0].EndRow = len(newdata)-1+offrow
            fcat.setPrintAreas( (pA[0],) )

    def changerPartie(self, force):
        form = self.feuilles("Classement").DrawPage.Forms.getByName("Formulaire")
        source = form.getByName("zlParties")
        try:
            n = source.SelectedItems[0]
        except IndexError:
            n = self.infos['derpartie'] or 1
            force = False
        x = source.BoundField.Value
        if n == 0: return
        if int(n) == x and not force: return
        source.commit()
        form.updateRow()
        # on modifie la source de la requête classement si pas déjà fait en amont
        if not force: self.changerSQLCalcPartie(n)
        # on cosmétise
        self.feuilles("Classement").getCellRangeByName("A5").setString("Partie %s" % n)
        # on recharge
        self.rafraichirClassements()
        self.categLLN()
        # self.doc.store()  # ne pas forcer la main à l'utilisateur

    def ouvrirClasseur(self,typepartie,form):
        '''Ouvre les classeurs classements en veillant à la mise à jour des résultats'''
        if utils.databasedocument.isModified(): utils.databasedocument.store()
        der = self.infos['derpartie']

        # quelques précautions
        if der == 0:
            self.msgbox("Aucune partie n'a encore été initiée !","Erreur",'warn')
            return
        else:
            tBord = form.Parent.getByName("tBord")
            tBord.moveToBookmark(der)
            if tBord.getString(tBord.findColumn("Résultats")) == "":
                if  der == 1 and typepartie == "Cumul":
                    self.msgbox("Aucun résultat n'a encore été encodé !","Erreur",'warn')
                    return

        # adaptation des données sources
        if typepartie == "Partie":
            # on modifie T_Infos pour simplifier affichage dans rapport
            self.executeQuery('UPDATE "T_Infos" SET "ClassementPartie" = %d' % der )
            # on modifie la requête source du classeur
            self.changerSQLCalcPartie(der)

        # ouverture proprement dite
        nouvURL = self.convertToUrl('{tooournoiPath}{nom}/{nom}_{}.ods'.format(typepartie,**self.infos))
        try:
            classeur = self.loadComponentFromURL(nouvURL)
        except (RuntimeException, IllegalArgumentException):
            modele = self.convertToUrl( self.infos['extPath'] + "modele_%s.ots" % typepartie)
            classeur = self.loadComponentFromURL(modele)
            classeur.storeAsURL( nouvURL, () )

        # hack pour éviter une erreur si classeur sauvegardé en mode Aperçu :
        ctrl = classeur.CurrentController
        frame = ctrl.Frame
        frame.ComponentWindow.setFocus()
        if not self.hasUnoInterface(ctrl, "com.sun.star.sheet.XSpreadsheetView"):
            self.executeDispatch(".uno:PrintPreview", frame=frame)

        # on recharge les données pour refléter les éventules changements
        if typepartie == "Partie":
            changerPartie(force=True, doc=classeur)
        else:  # typepartie == "Cumul"
            rafraichirClassements(categ=True, doc=classeur)
        classeur.store()

    def changerSQLCalcPartie(self, partie):
        ''' on modifie la requête source du classeur par regex : il s'agit de remplacer
        tous les n° Tx, Rx, Px sauf pour la portion <OR "P1"> qui doit rester P1
        La fonction attend le numéro de partie en paramètre'''
        import re
        requete = utils.connexion.Queries.getByName("Calc_partie")
        requete.Command = re.sub( '(?!OR "P1)(..."[T|P|R])([0-9]+)', r'\g<1>%d' % partie, requete.Command )

    def protectSheets(self):
        for f in self.iterUno(self.doc.Sheets): f.protect("")

    def unprotectSheets(self):
        for f in self.iterUno(self.doc.Sheets): f.unprotect("")

    def exportCSV(self, model):
        """
        @model : "1" pour classement complet, "2" pour classement par catégorie
        penser à modifier la signature de la routine "utils.filePicker" en conséquence : 
            utils.filePicker(filters=(("Tous les fichiers (*.*)", "*.*"),),
                             multi=False,
        ->                   mode=FILEOPEN_SIMPLE)
        """
        # from com.sun.star.ui.dialogs.TemplateDescription import FILESAVE_SIMPLE
        # O.filePicker(filters=(("Fichiers CSV (*.csv)", "*.csv"),), mode=FILESAVE_SIMPLE)
        filepath = os.path.dirname(self.convertFromUrl(self.doc.URL))
        if model=="1":
            feuille = self.doc.CurrentController.ActiveSheet
            pa = feuille.PrintAreas[0]
            exportrange = feuille.getCellRangeByPosition(pa.StartColumn,9,pa.EndColumn,pa.EndRow)
            filename = "%s_%s.csv" % (feuille.getCellByPosition(0,2).String, 
                            feuille.getCellByPosition(0,4).String)
            self.dataarray2csv(os.path.join(filepath,filename), exportrange.DataArray)
        elif model=="2":
            cropindex = 8+self.infos['derpartie']*2
            data = [list(x[:cropindex]) for x in self.dbR.getByName(self.plageRef).ReferredCells.getDataArray()] # (()) -> [[]] pour traitement
            for cat in self.CAT:
                newdata = []
                lasttop = 0
                for row in data[2:]:
                    if row[4] == cat[0]:
                        if lasttop == 0:
                            row[0] = "1"
                        elif row[7] == lasttop:
                            row[0] = '-'
                        else:
                            row[0] = str(len(newdata)+1)
                        newdata.append(row)
                        lasttop = row[7]
                if newdata:
                    newdata = data[:2]+newdata
                    classement = self.plageRef.title()
                    if classement=="Cumul":
                        classement += " après %d %s" % (self.infos['derpartie'], self.infos['derpartie']>1 and "manches" or "manche")
                    else:
                        classement += " %d" % self.infos['derpartie']
                    filename = "%s_%s_%s.csv" % (self.infos['nom'],classement, cat[1].upper())
                    self.dataarray2csv(os.path.join(filepath,filename), newdata)

    def exportPDF(self, model):
        from uno import Any as unoAny
        feuille = self.doc.CurrentController.ActiveSheet
        aFilterData = self.createPropertyValues(Selection=feuille)
        filepath = os.path.dirname(self.convertFromUrl(self.doc.URL))
        if model=="1":
            filename = "%s_%s.pdf" % (feuille.getCellByPosition(0,2).String, 
                            feuille.getCellByPosition(0,4).String)
            url = self.convertToUrl(os.path.join(filepath,filename))
            props = self.createPropertyValues(FilterName="calc_pdf_Export",
                                              URL=url,
                                              Overwrite=True,
                                              # Villeroy : "Arrays of non-trivial types need to be composed
                                              #    using a helping runtime function uno.Any(strUNOType, x)"
                                              FilterData=unoAny("[]com.sun.star.beans.PropertyValue", aFilterData))
            self.executeDispatch(".uno:ExportDirectToPDF", props)
        elif model=="2":
            self.unprotectSheets()
            categories = self.doc.NamedRanges.categories.ReferredCells.getDataArray()
            cell = feuille.getCellByPosition(2,0)
            for cat in categories:
                if cat[1]>0:
                    cell.setString(cat[2])
                    self.categLLN()
                    filename = "%s_%s.pdf" % (feuille.getCellByPosition(0,2).String, 
                                    feuille.getCellByPosition(0,3).String)
                    url = self.convertToUrl(os.path.join(filepath,filename))
                    props = self.createPropertyValues(FilterName="calc_pdf_Export",
                                                      URL=url,
                                                      Overwrite=True,
                                                      # Villeroy : "Arrays of non-trivial types need to be composed
                                                      #    using a helping runtime function uno.Any(strUNOType, x)"
                                                      FilterData=unoAny("[]com.sun.star.beans.PropertyValue", aFilterData))
                    self.executeDispatch(".uno:ExportDirectToPDF", props)
            self.protectSheets()


# ---------------------------------------
def rafraichirClassements(ev=None, doc=None, categ=False):
    C = Classements(doc)
    C.unprotectSheets()
    C.rafraichirClassements()
    C.calculerCatSer()
    if categ:
        C.categLLN()
        nbcol = 8+C.infos['maxpartie']*2 
        C.feuilles('CATÉGORIES').Columns.removeByIndex(nbcol,1024-nbcol)
    C.protectSheets()

def changerPartie(ev=None, doc=None, force=False):
    C = Classements(doc)
    C.unprotectSheets()
    C.changerPartie(force)
    C.calculerCatSer()
    C.protectSheets()
