    # -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sys
import os.path
from com.sun.star.sdbc import SQLException
from com.sun.star.uno import RuntimeException
from com.sun.star.lang import IllegalArgumentException

import utils
if sys.version_info.major < 3:
    from duplitop import Duplitop
else:
    from duplitop3 import Duplitop


class Duplitour(utils.Tournoi):
    def __init__(self):
        super(Duplitour,self).__init__()

    def importPartie(self, event=None):
        '''Extrait les données des fichiers duplitop à l'aide du module
        duplitop.py de Hughes Damry. Les données sont enregistrées dans
        les tables "T_duplitableurs_xxx" et "T_dupliparties_xxx", où <xxx>
        est le nom du tournoi chargé dans tOOournoi au moment de l'appel.'''
        
        def negatize(points, tops):
            '''Si top, renvoie NULL, sinon renvoie le négatif.'''
            return (x==y and 'NULL' or x-y for x,y in zip(points, tops))
        
        # récupère le numéro de la partie en cours (cosmétique, pour le dialogue)
        derPartie = self.infos['derpartie']
        # petit dialogue pour le choix de la partie et des fichiers
        dlg = self.createTooournoiDialog('dialogue8')
        list = dlg.getControl('list')
        list.addItems( tuple(range(1,derPartie+1)), 0 )
        list.selectItem(derPartie, True)
        if not dlg.execute(): return
        num, ecrase = list.SelectedItem, dlg.getControl('check').State
        dlg.dispose()
        # récupère les chemins des fichiers
        files = self.filePicker(filters=(("Fichiers Duplitop", "*.HOP;*.CLP;*.INP;*.TOP;*.ENP;*.TRP;*.PAP"),
                                         ("Tous les fichiers (*.*)", "*.*")), 
                                multi=True)
        if not files : return
        # défini le nom de la table à partir du nom de la compétition
        t_dupliparties = 'T_dupliparties_%s' % (self.infos['nom'])
        t_duplitableurs = 'T_duplitableurs_%s' % (self.infos['nom'])
        # on crée les tables si nécessaire
        log = []
        if not utils.connexion.Tables.hasByName(t_dupliparties):
            sSQL = ('CREATE TABLE "%s" ("ID" INT GENERATED BY DEFAULT AS IDENTITY,\n'
                    '\t"PARTIE" INT, "COUP" INT, "TIRAGE" VARCHAR_IGNORECASE (15),\n'
                    '\t"MOT" VARCHAR_IGNORECASE (15),\n'
                    '\t"REF" VARCHAR_IGNORECASE (5), "POINTS" INT, "CUMUL" INT,\n'
                    '\tCONSTRAINT "c_%s" UNIQUE ("PARTIE","COUP"));\n'
                      % (t_dupliparties, t_dupliparties) )
            sSQL += ('CREATE TABLE "%s" ("ID" INT GENERATED BY DEFAULT AS IDENTITY,\n'
                     '\t"PARTIE" INT, "TABLE" INT, "FIXE" BOOLEAN, "COUP" INT,\n'
                     '\t"POINTS" INT, "NEG" INT, "SOLO" BOOLEAN, "CUMUL" INT,\n'
                     '\tCONSTRAINT "c_%s" UNIQUE ("PARTIE","TABLE","FIXE","COUP"));'
                       % (t_duplitableurs, t_duplitableurs) )
            try:
                self.stmt.executeUpdate(sSQL)
                log.append('Création des tables réussies :\n\n%s\n' % sSQL)
                utils.connexion.Tables.refresh()
            except SQLException as e:
                self.msgbox(e.Message.encode('latin-1'), boxtype='error')
        # insertion des données de la partie après avoir vérifié si partie déjà enregistrée ou non
        rs = self.stmt.executeQuery('SELECT "PARTIE" FROM "%s" WHERE "PARTIE" = %s' % (t_dupliparties,num))
        if not rs.next():  # aucun enregistrement avec ce numéro de partie
            fichier = self.convertFromUrl(files[0])
            d=Duplitop('',fichier)
            d.open()
            d.read()
            d.parsefile()
            sSQL = ''
            for c in d.coups:
                sSQL += ('INSERT INTO "%s" VALUES (NULL, %s, %d, \'%s\', \'%s\', \'%s\', %d, %d);\n'
                         % (t_dupliparties, num, c.num, c.tirage,
                            c.mot, c.alphanum(), c.compte, c.CumulPartie) )
            try:
                self.stmt.executeUpdate(sSQL)
                utils.connexion.Tables.refresh()
            except SQLException as e:
                self.msgbox(e.Message.encode('latin-1'), boxtype='error')
        # on insère les données joueurs
        for file in files:
            fichier = self.convertFromUrl(file)
            d=Duplitop('',fichier)
            d.open()
            d.read()
            d.parsefile()
            tops = [x[4] for x in d.partie()]
            aSQL = []
            for j in (x for x in d.joueursdic.values() if not x.tablevide()):
                if ecrase:
                    aSQL.append('DELETE FROM "%s" WHERE "PARTIE" = %s AND "TABLE" = %s;\n'
                                  % (t_duplitableurs, num, j.table))
                # conversion en tuple ajoutée pour compatibilité avec python3
                coups = tuple(zip(j.ptscoups, negatize(j.ptscoups,tops), j.ptssolo, j.cumuls))
                for c in range(d.NC):
                    aSQL.append('INSERT INTO "%s" VALUES(NULL,%s,%s,%d,%d,%d,%s,%d,%d);\n'
                                  % ((t_duplitableurs, num, j.table,
                                     j.tablefixe, c+1) + coups[c]) )
            try:
                self.stmt.executeUpdate(''.join(aSQL))
                log.append("%s : traité avec succès." % os.path.basename(fichier))
            except SQLException as e:
                if "Violation of unique constraint" in e.Message:
                    log.append("%s : traitement interrompu.\n\tErreur :\n\t%s"
                                    % (os.path.basename(fichier), "Ce fichier a déjà été importé.\n"
                                       "\tCochez l'option 'écraser' pour réimporter le ficher.\t"))
                else:
                    log.append("%s : traitement interrompu.\n\tErreur :\n\t<< %s >>"
                                    % (os.path.basename(fichier), e.Message))
        self.msgbox('\n------------------\n'.join(log).encode('latin-1'))

    def statsParPartie(self,partie=1):
        """Extraction et traitement des données récupérées à l'aide de la
        fonction importPartie. L'essentiel du traitement se fait en SQL sur
        les tables "T_duplitableurs_xxx" et "T_dupliparties_xxx", où <xxx> est
        le nom du tournoi chargé au moment de l'appel."""

        if partie==0: return
        def modifDataRange(nom, SQL):
                RC = self.doc.DatabaseRanges.getByName(nom).ReferredCells
                desc = RC.createImportDescriptor(False)
                desc[2].Value = SQL
                RC.doImport(desc)

        def setGrille():
            vRef = "ABCDEFGHIJKLMNO" # facilite le transposage grâce à l'index de la référence
            grille = self.doc.NamedRanges.grille.ReferredCells
            grille.clearContents(31)
            DA = [list(x) for x in grille.DataArray]  # (()) -> [[]] pour traitement
            mots = self.doc.DatabaseRanges.Partie.ReferredCells.DataArray
            for m in mots[1:]:
                mot, ref = m[2:4]
                if ref[0] in vRef:  # si commence par lettre -> mot horizontal
                    row = vRef.index(ref[0])
                    col = int(ref[1:])-1
                    DA[row][col:col+len(mot)] = mot
                else:  # sinon -> mot vertical
                    row = vRef.index(ref[-1])
                    col = int(ref[:-1])-1
                    for x in range(len(mot)):
                        DA[row+x][col] = mot[x]
            grille.setDataArray(tuple(tuple(d) for d in DA))  # [[]] -> (()) pour renvoi vers UNO

        self.infos['derpartie'] = partie
        self.doc.NamedRanges.partie.ReferredCells.setValue(partie)
        for dp in self.iterUno(self.doc.DrawPages):
            dp.Forms.getByIndex(0).getByName("zlPartie").SelectedItems=(partie,)
        
        if utils.connexion.Tables.hasByName("T_dupliparties_" + self.infos['nom']):
            p_encodee = self.executeQuery('SELECT * FROM "T_dupliparties_%(nom)s" WHERE "PARTIE" = %(derpartie)d'
                                    % self.infos).next()
            if p_encodee:
                # 1 : la partie
                SQL = ('SELECT "COUP" AS " ", "TIRAGE", "MOT", "REF" AS "Réf.", "POINTS" AS "Pts" '
                       'FROM "T_dupliparties_%(nom)s" WHERE "PARTIE" = %(derpartie)d' % self.infos)
                modifDataRange("Partie", SQL)

                # 2 : la grille
                setGrille()

                # 3 : les solos par séries
                SQL = ('SELECT "S"."COUP", "S"."SERIE", "C"."Joueur" AS "SOLISTE", "MOT" '
                       '    FROM ( SELECT "DT"."PARTIE", "DT"."COUP", LEFT( "R"."Série", 1 ) AS "SERIE", COUNT( "DP"."POINTS" ) "TOPEURS", "MOT" '
                       '           FROM "T_duplitableurs_%(nom)s" "DT", "T_dupliparties_%(nom)s" "DP", "R_Cumul" "R" '
                       '           WHERE "DT"."NEG" IS NULL AND "DP"."PARTIE" = %(derpartie)d '
                       '                AND "DP"."PARTIE" = "DT"."PARTIE" '
                       '                AND "DP"."COUP" = "DT"."COUP" '
                       '                AND "DT"."TABLE" = "R"."T%(derpartie)d" '
                       '           GROUP BY "DP"."PARTIE", "DP"."COUP", "DT"."NEG", LEFT( "R"."Série", 1 ), "MOT" '
                       '           HAVING COUNT( "DP"."POINTS" ) = 1 ) "S", '
                       '           "T_duplitableurs_%(nom)s" "D", "R_Cumul" "C" '
                       '    WHERE "D"."NEG" IS NULL AND "S"."COUP" = "D"."COUP" AND "S"."PARTIE" = "D"."PARTIE" '
                       '        AND "S"."SERIE" = "C"."S" AND "D"."TABLE" = "C"."T%(derpartie)d" '
                       '    ORDER BY "S"."PARTIE", "S"."COUP", "S"."SERIE"' % self.infos)
                modifDataRange("SolosParPartie", SQL)

                # 4 : les topeurs
                SQL = ('SELECT "MOT", SUM("TOPS") AS "TOPEURS"'
                       'FROM (SELECT "T"."COUP" AS "COUP", "MOT", CASE WHEN "NEG" IS NULL OR "SOLO" = TRUE THEN 1 ELSE 0 END AS "TOPS"'
                       '      FROM "T_duplitableurs_%(nom)s" "T", "T_dupliparties_%(nom)s" "P"'
                       '      WHERE "T"."PARTIE" = "P"."PARTIE" AND "T"."COUP" = "P"."COUP"'
                       '            AND "PARTIE" = %(derpartie)d)'
                       'GROUP BY "MOT", "COUP" ORDER BY "COUP"' % self.infos)
                modifDataRange("Topeurs", SQL)
                
                # 5 : rafraîchir le nom du tournoi
                self.doc.DatabaseRanges.getByName('NomTournoi').refresh()
                self.doc.store()

    def statsSerCat(self, type):
        '''Fonction à l'état de test.'''
        distrib = {'1':0,'2':0,'3':0,'4':0,'5':0,'6':0,'7':0,'P':0,'B':0,'C':0,'J':0,'E':0,'S':0,'V':0,'D':0}
        SQLs = {'zeros': ('SELECT "S", COUNT("POINTS") FROM (SELECT LEFT("Ser",1) "S", "POINTS" FROM "R_Duplitableur" WHERE "POINTS" = 0) GROUP BY "S"',
                          'SELECT "Cat", COUNT("POINTS") FROM (SELECT "Cat", "POINTS" FROM "R_Duplitableur" WHERE "POINTS" = 0) GROUP BY "Cat"'),
                'moyneg': ('SELECT "S", SUM("MOY") FROM( SELECT LEFT("Ser",1) "S", SUM("NEG")/COUNT("Ser") "MOY" FROM "R_Duplitableur" GROUP BY LEFT("Ser",1), "Coup") GROUP BY "S"',
                           'SELECT "Cat", SUM("MOY") FROM( SELECT "Cat", SUM("NEG")/COUNT("Cat") "MOY" FROM "R_Duplitableur" GROUP BY "Cat", "Coup") GROUP BY "Cat"'),
                'nbjou': ('SELECT LEFT("Ser",1) "S", COUNT("Ser") FROM "R_Duplitableur" GROUP BY "S", "Coup", "PARTIE" HAVING ("Coup"=1 AND "PARTIE"=1)',
                          'SELECT "Cat", COUNT("Cat") FROM "R_Duplitableur" GROUP BY "Cat", "Coup", "PARTIE" HAVING ("Coup"=1 AND "PARTIE"=1)')
               }
        t_dupliparties = 'T_dupliparties_%s' % self.infos['nom']
        t_duplitableurs = 'T_duplitableurs_%s' % self.infos['nom']
        # vérifions quand même que les données ont été importées
        if not utils.connexion.Tables.hasByName(t_duplitableurs): return
        for SQL in SQLs[type]:
            # on récupère les zéros par séries
            rowset = self.stmt.executeQuery(SQL)
            while rowset.next():
                distrib[rowset.getString(1)] = rowset.getString(2)
        return repr(distrib)

    def ouvrirClasseur(self):
        if utils.connexion.Tables.hasByName("T_dupliparties_%(nom)s" % self.infos):
            nouvURL = self.convertToUrl('%(tooournoiPath)s%(nom)s/%(nom)s_Stats.ods' % self.infos)
            try:
                self.doc = self.loadComponentFromURL(nouvURL)
            except (RuntimeException, IllegalArgumentException):
                modele = self.convertToUrl( self.infos['extPath'] + "modele_Stats.ots")
                self.doc = self.loadComponentFromURL(modele)
                self.doc.storeAsURL( nouvURL, () )
                f = self.doc.CurrentController.ActiveSheet
                form = f.DrawPage.Forms.getByName("Formulaire")
                source = form.getByName("zlPartie")
                self.statsParPartie(source.SelectedItems[0])
            return True
        self.msgbox("Importez d'abord les fichiers Duplitop.\t","Données manquantes","warn")
        return False
